@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008-2012 NVIDIA Corporation
+ *  Copyright 2008-2013 NVIDIA Corporation
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -81,6 +81,7 @@ namespace thrust
  *  \endcode
  */
 template<typename DerivedPolicy, typename InputIterator, typename OutputIterator>
+__host__ __device__
   OutputIterator copy(const thrust::detail::execution_policy_base<DerivedPolicy> &exec,
                       InputIterator first,
                       InputIterator last,
@@ -133,6 +134,7 @@ template<typename DerivedPolicy, typename InputIterator, typename OutputIterator
  *  \see thrust::copy
  */
 template<typename DerivedPolicy, typename InputIterator, typename Size, typename OutputIterator>
+__host__ __device__
   OutputIterator copy_n(const thrust::detail::execution_policy_base<DerivedPolicy> &exec,
                         InputIterator first,
                         Size n,
@@ -239,8 +241,9 @@ template<typename InputIterator, typename Size, typename OutputIterator>
 
 
 /*! This version of \p copy_if copies elements from the range <tt>[first,last)</tt>
- *  to a range beginning at \ presult, except that any element which causes \p pred
- *  to be \p pred to be \c false is not copied.
+ *  to a range beginning at \p result, except that any element which causes \p pred
+ *  to be \c false is not copied. \p copy_if is stable, meaning that the relative
+ *  order of elements that are copied is unchanged.
  *
  *  More precisely, for every integer \c n such that <tt>0 <= n < last-first</tt>,
  *  \p copy_if performs the assignment <tt>*result = *(first+n)</tt> and \p result
@@ -294,6 +297,7 @@ template<typename InputIterator, typename Size, typename OutputIterator>
  *  \see \c remove_copy_if
  */
 template<typename DerivedPolicy, typename InputIterator, typename OutputIterator, typename Predicate>
+__host__ __device__
   OutputIterator copy_if(const thrust::detail::execution_policy_base<DerivedPolicy> &exec,
                          InputIterator first,
                          InputIterator last,
@@ -303,8 +307,9 @@ template<typename DerivedPolicy, typename InputIterator, typename OutputIterator
 
 
 /*! This version of \p copy_if copies elements from the range <tt>[first,last)</tt>
- *  to a range beginning at \ presult, except that any element which causes \p pred
- *  to be \p pred to be \c false is not copied.
+ *  to a range beginning at \p result, except that any element which causes \p pred
+ *  to \c false is not copied. \p copy_if is stable, meaning that the relative
+ *  order of elements that are copied is unchanged.
  *
  *  More precisely, for every integer \c n such that <tt>0 <= n < last-first</tt>,
  *  \p copy_if performs the assignment <tt>*result = *(first+n)</tt> and \p result
@@ -363,7 +368,8 @@ template<typename InputIterator,
 
 /*! This version of \p copy_if copies elements from the range <tt>[first,last)</tt>
  *  to a range beginning at \p result, except that any element whose corresponding stencil
- *  element causes \p pred to be \c false is not copied.
+ *  element causes \p pred to be \c false is not copied. \p copy_if is stable, meaning
+ *  that the relative order of elements that are copied is unchanged.
  *
  *  More precisely, for every integer \c n such that <tt>0 <= n < last-first</tt>,
  *  \p copy_if performs the assignment <tt>*result = *(first+n)</tt> and \p result
@@ -422,6 +428,7 @@ template<typename InputIterator,
  *  \see \c remove_copy_if
  */
 template<typename DerivedPolicy, typename InputIterator1, typename InputIterator2, typename OutputIterator, typename Predicate>
+__host__ __device__
   OutputIterator copy_if(const thrust::detail::execution_policy_base<DerivedPolicy> &exec,
                          InputIterator1 first,
                          InputIterator1 last,
@@ -432,7 +439,8 @@ template<typename DerivedPolicy, typename InputIterator1, typename InputIterator
 
 /*! This version of \p copy_if copies elements from the range <tt>[first,last)</tt>
  *  to a range beginning at \p result, except that any element whose corresponding stencil
- *  element causes \p pred to be \c false is not copied.
+ *  element causes \p pred to be \c false is not copied. \p copy_if is stable, meaning
+ *  that the relative order of elements that are copied is unchanged.
  *
  *  More precisely, for every integer \c n such that <tt>0 <= n < last-first</tt>,
  *  \p copy_if performs the assignment <tt>*result = *(first+n)</tt> and \p result
