@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008-2012 NVIDIA Corporation
+ *  Copyright 2008-2013 NVIDIA Corporation
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -19,6 +19,8 @@
 #include <thrust/detail/config.h>
 #include <thrust/system/cuda/detail/execution_policy.h>
 #include <thrust/detail/execute_with_allocator.h>
+#include <thrust/system/cuda/detail/execute_on_stream.h>
+#include <thrust/detail/allocator/allocator_traits.h>
 
 namespace thrust
 {
@@ -35,10 +37,20 @@ struct par_t : thrust::system::cuda::detail::execution_policy<par_t>
   par_t() : thrust::system::cuda::detail::execution_policy<par_t>() {}
 
   template<typename Allocator>
-    thrust::detail::execute_with_allocator<Allocator, thrust::system::cuda::detail::execution_policy>
-      operator()(Allocator &alloc) const
+  __host__ __device__
+  typename thrust::detail::enable_if<
+    thrust::detail::is_allocator<Allocator>::value,
+    thrust::detail::execute_with_allocator<Allocator, execute_on_stream_base>
+  >::type
+    operator()(Allocator &alloc) const
   {
-    return thrust::detail::execute_with_allocator<Allocator, thrust::system::cuda::detail::execution_policy>(alloc);
+    return thrust::detail::execute_with_allocator<Allocator, execute_on_stream_base>(alloc);
+  }
+
+  __host__ __device__
+  inline execute_on_stream on(const cudaStream_t &stream) const
+  {
+    return execute_on_stream(stream);
   }
 };
 
@@ -46,7 +58,11 @@ struct par_t : thrust::system::cuda::detail::execution_policy<par_t>
 } // end detail
 
 
+#ifdef __CUDA_ARCH__
+static const __device__ detail::par_t par;
+#else
 static const detail::par_t par;
+#endif
 
 
 } // end cuda
