@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008-2012 NVIDIA Corporation
+ *  Copyright 2008-2013 NVIDIA Corporation
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -19,9 +19,11 @@
 #include <thrust/detail/config.h>
 
 // the purpose of this header is to #include the tabulate.h header
-// of the host and device systems. It should be #included in any
+// of the sequential, host, and device systems. It should be #included in any
 // code which uses adl to dispatch tabulate
 
+#include <thrust/system/detail/sequential/tabulate.h>
+
 #define __THRUST_HOST_SYSTEM_TABULATE_HEADER <__THRUST_HOST_SYSTEM_ROOT/detail/tabulate.h>
 #include __THRUST_HOST_SYSTEM_TABULATE_HEADER
 #undef __THRUST_HOST_SYSTEM_TABULATE_HEADER
