@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008-2012 NVIDIA Corporation
+ *  Copyright 2008-2013 NVIDIA Corporation
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -19,9 +19,11 @@
 #include <thrust/detail/config.h>
 
 // the purpose of this header is to #include the temporary_buffer.h header
-// of the host and device systems. It should be #included in any
+// of the sequential, host, and device systems. It should be #included in any
 // code which uses adl to dispatch get_temporary_buffer or return_temporary_buffer
 
+#include <thrust/system/detail/sequential/temporary_buffer.h>
+
 #define __THRUST_HOST_SYSTEM_TEMPORARY_BUFFER_HEADER <__THRUST_HOST_SYSTEM_ROOT/detail/temporary_buffer.h>
 #include __THRUST_HOST_SYSTEM_TEMPORARY_BUFFER_HEADER
 #undef __THRUST_HOST_SYSTEM_TEMPORARY_BUFFER_HEADER
